import { addCompKey, addLayoutKey } from './metaHandle'

/**
 * @param h vue 在 render 方法里的 createElement
 * @param layout 布局配置
 * @returns {[VNode]}
 */
export function renderLayout(h, layout) {
    addLayoutKey(layout)
    return h(
        'comp-view',
        {
            props: {
                compConfig: {
                    ...layout,
                    isLayout: true
                }
            }
        },
        [
            h(
                'comp-layout',
                {
                    props: {
                        type: layout.type,
                        components: layout.components,
                        layoutKey: layout._key,
                        leftSpan: layout.leftSpan
                    }
                }
            )
        ]
    )
}

export function renderHeaderSlot(h, items) {
    if (items && items.length) {
        return items.map((slot) => {
            addCompKey(slot);
            return h(
                'comp-view',
                {
                    slot: `header-${slot.position}`,
                    props: {
                        isClear: false,
                        compConfig: slot
                    }
                },
                [
                    h(
                        slot.type,
                        {
                            props: {
                                ...slot.props,
                                designtime: true
                            }
                        }
                    )
                ]
            )
        });
    }

    return [];
}

/**
 * @param h vue 在 render 方法里的 createElement
 * @param component 组件配置
 * @param commonMeta 公共配置
 * @returns {[VNode]}
 */
export function renderComponent(h, component, commonMeta) {
    addCompKey(component);
    return h(
        'comp-view',
        {
            props: {
                compConfig: component,
            }
        },
        [
            h(
                component.type,
                {
                    class: component.class,
                    style: component.style,
                    attrs: component.attrs,
                    props: {
                        ...component.props,
                        commonMeta,
                        designtime: true
                    },
                    domProps: component.domProps,
                    on: component.on,
                    nativeOn: component.nativeOn,
                    directives: component.directives,
                    scopedSlots: component.scopedSlots,
                    slot: component.slot,
                    key: component.key,
                    ref: component.ref
                }
            )
        ]
    )
}

/**
 * @param h vue 在 render 方法里的 createElement
 * @param view 页面元数据
 * @param commonMeta
 * @returns {[VNode]}
 */
export function renderViewBody(h, view, commonMeta) {
    const content = view.body.content;
    return content && content.items && content.items.map((item) => {
        if (item.type === 'horizontal' || item.type === 'vertical') {
            return renderLayout(h, item)
        }
        return renderComponent(h, item, commonMeta)
    })
}

/**
 * @param h vue 在 render 方法里的 createElement
 * @param view 页面元数据
 * @returns {[VNode]}
 */
export function renderViewFooter(h, view) {
    const footer = view.body.footer;
    if (!footer) return null;
    return renderComponent(h, { slot: 'footer', ...footer })
}

/**
 * @param h vue 在 render 方法里的 createElement
 * @param meta 页面元数据
 * @param tabs
 * @param tabIndex
 * @returns {[VNode]}
 */
export function renderTabFooter(h, meta, tabs, tabIndex) {
    return h(
        'comp-view',
        {
            slot: 'footer',
            props: {
                compConfig: {
                    type: 'tabmenu',
                    props: {
                        menus: tabs.items,
                        tabbarStyle: tabs.style
                    }
                },
                isClear: false
            }
        },
        [
            h(
                'tabmenu',
                {
                    props: {
                        menus: tabs.items,
                        activeIndex: tabIndex,
                        tabbarStyle: tabs.style,
                        normalColor: meta.common.style.second_color
                    }
                }
            )
        ]
    )
}

/**
 * @param h vue 在 render 方法里的 createElement
 * @param view 页面元数据
 * @param appmeta
 * @param tabs
 * @param tabIndex
 * @returns {[VNode]}
 */
export function renderView(h, view, appmeta, tabs, tabIndex) {
    const _header = view.body.header;
    const style = appmeta.common ? appmeta.common.style : {};
    const pageStyle = {
        backgroundColor: style.global_bg_color,
        ...view.style
    };
    const headerStyle = {
        backgroundColor: style.primary_color,
        ..._header.style
    };
    const footerStyle = {
        backgroundColor: style.tab_bar_bg_color,
    };

    let hasSlotLeft;
    if (!_header.items || !_header.items.length) {
        hasSlotLeft = false
    } else {
        hasSlotLeft = !!_header.items.find(item => item.position === 'left')
    }
    return h(
        'page',
        {
            props: {
                title: view.body.header.title || '',
                showHeader: !_header.hide,
                showHeaderBack: !hasSlotLeft && false,
                pageStyle,
                headerStyle,
                footerStyle
            }
        },
        [
            h('div', { class: 'x-page-content' }, [
                h('comp-sortable', null, renderViewBody(h, view, appmeta.common))
            ]),
            tabs && tabs.items && tabs.items.length > 0
                ? renderTabFooter(h, appmeta, tabs, tabIndex)
                : renderViewFooter(h, view)
        ].concat(renderHeaderSlot(h, _header.items))
    );
}
