/**
 * @param h vue 在 render 方法里的 createElement
 * @param layout 布局配置
 * @returns {[VNode]}
 */
export function renderLayout(h, layout) {
    return h('layout',
        {
            props: {
                type: layout.type,
                leftSpan: layout.leftSpan,
                components: layout.components
            }
        }
    )
}

/**
 * render header slot 组件
 * @param {*} h
 * @param {*} items
 */
export function renderHeaderSlot(h, items) {
    if (items && items.length) {
        return items.map(slot => h(
            slot.type,
            {
                slot: `header-${slot.position}`,
                props: {
                    ...slot.props,
                    designtime: false
                }
            })
        );
    }

    return [];
}

/**
 * @param h vue 在 render 方法里的 createElement
 * @param component 组件配置
 * @param commonMeta 全局样式
 * @returns {[VNode]}
 */
export function renderComponent(h, component, commonMeta) {
    return h(component.type,
        {
            class: component.class,
            style: component.style,
            attrs: component.attrs,
            props: {
                ...component.props,
                commonMeta,
                designtime: false
            },
            domProps: component.domProps,
            on: component.on,
            nativeOn: component.nativeOn,
            directives: component.directives,
            scopedSlots: component.scopedSlots,
            slot: component.slot,
            key: component.key,
            ref: component.ref
        }
    )
}

/**
 * @param h vue 在 render 方法里的 createElement
 * @param view 页面元数据
 * @param commonMeta 全局样式
 * @returns {[VNode]}
 */
export function renderViewBody(h, view, commonMeta) {
    const content = view.body.content;
    return content.items.map((item) => {
        if (item.type === 'horizontal' || item.type === 'vertical') {
            return renderLayout(h, item)
        }
        return renderComponent(h, item, commonMeta)
    })
}

/**
 * @param h vue 在 render 方法里的 createElement
 * @param view 页面元数据
 * @returns {[VNode]}
 */
export function renderViewFooter(h, view) {
    const footer = view.body.footer;
    if (!footer) return null;
    return renderComponent(h, { slot: 'footer', ...footer })
}

/**
 * @param h vue 在 render 方法里的 createElement
 * @param meta 页面元数据
 * @param tabIndex tabIndex
 * @returns {[VNode]}
 */
export function renderTabFooter(h, meta, tabIndex) {
    return h(
        'tabmenu',
        {
            slot: 'footer',
            props: {
                menus: meta.tabs.items,
                activeIndex: tabIndex,
                tabbarStyle: meta.tabs.style,
                normalColor: meta.common.style.second_color
            }
        }
    )
}

/**
 * @param h vue 在 render 方法里的 createElement
 * @param view 页面元数据
 * @param appmeta app元数据
 * @returns {[VNode]}
 */
export function renderView(h, view, appmeta) {
    let isTabView = false;
    let tabIndex = 0;
    if (appmeta.tabs && appmeta.tabs.items) {
        const viewTab = appmeta.tabs.items.find(item => item.href.name === view.name);
        if (viewTab) {
            isTabView = true;
            tabIndex = appmeta.tabs.items.map(tab => tab.href && tab.href.name).indexOf(view.name)
        }
    }

    const _header = view.body.header;
    const style = appmeta.common ? appmeta.common.style : {};
    const pageStyle = {
        backgroundColor: style.global_bg_color,
        ...view.style
    };
    const headerStyle = {
        backgroundColor: style.primary_color,
        ..._header.style
    };
    const footerStyle = {
        backgroundColor: style.tab_bar_bg_color,
    };

    let hasSlotLeft;
    if (!_header.items || !_header.items.length) {
        hasSlotLeft = false
    } else {
        hasSlotLeft = !!_header.items.find(item => item.position === 'left')
    }

    return h(
        'page',
        {
            props: {
                title: _header.title || '',
                showHeader: !_header.hide,
                showHeaderBack: !hasSlotLeft && !isTabView,
                pageStyle,
                headerStyle,
                footerStyle
            }
        },
        [
            h('div', { class: 'x-page-content' }, renderViewBody(h, view, appmeta.common)),
            isTabView ? renderTabFooter(h, appmeta, tabIndex) : renderViewFooter(h, view)
            // renderViewFooter(h, view)
        ].concat(renderHeaderSlot(h, _header.items))
    )
}
